-- Addon creation and initialization

DiminishingReturns = CreateFrame("Frame")
local addon = DiminishingReturns
LibStub('LibAdiEvent-1.0').Embed(addon)

local DEFAULT_CONFIG = {
	autoCategories = true,
	categories = { ['*'] = true },
}
addon.DEFAULT_CONFIG = DEFAULT_CONFIG

local function OnLoad(self, event, name)
	if name:lower() ~= "diminishingreturns" then return end
	self:UnregisterEvent('ADDON_LOADED', OnLoad)
	OnLoad = nil
	
	local db = LibStub('AceDB-3.0'):New("DiminishingReturnsDB", {profile=DEFAULT_CONFIG})
	--db.RegisterCallback(self, 'OnProfileChanged', 'RequireUpdate')
	--db.RegisterCallback(self, 'OnProfileCopied', 'RequireUpdate')
	--db.RegisterCallback(self, 'OnProfileReset', 'RequireUpdate')
	self.db = db
	
	-- Optional LibDualSpec-1.0 support
	local lds = LibStub('LibDualSpec-1.0', true)
	if lds then
		self.LibDualSpec = lds
		lds:EnhanceDatabase(db, "Diminishing Returns")
	end
	
	-- Load supporting addon for already loaded, supported addons
	local function LoadSupportAddons()
		for index = 1, GetNumAddOns() do
			local name = GetAddOnMetadata(index, "X-DiminishingReturns-AddonSupport")
			if name then
				if IsAddOnLoaded(name) then
					LoadAddOn(index)
				else
					local _, _, _, enabled, loadable = GetAddOnInfo(name)
					if enabled and loadable then
						local index, name = index, name:lower()
						local loader = function(_, _, loaded)
							if loaded and loaded:lower() == name then
								LoadAddOn(index)
								addon:UnregisterEvent('ADDON_LOADED', loader)
							end
						end
						addon:RegisterEvent('ADDON_LOADED', loader)
					end
				end
			end
		end
		addon:UnregisterEvent('PLAYER_LOGIN', LoadSupportAddons)
	end

	if IsLoggedIn() then
		LoadSupportAddons()
	else
		addon:RegisterEvent('PLAYER_LOGIN', LoadSupportAddons)
	end

	addon:RegisterBlizzardFrames()
end
addon:RegisterEvent('ADDON_LOADED', OnLoad)

-- Test mode
function addon:SetTestMode(mode)
	self.testMode = mode
	addon:TriggerMessage('SetTestMode', self.testMode)
end

SLASH_DRTEST1 = "/drtest"
SlashCmdList.DRTEST = function()
	addon:SetTestMode(not addon.testMode)
end
